==> ifrit_3.0.5.orig.tar.gz: ifrit-3.0.5/core/isolidviewsubject.cpp <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/


#include "isolidviewsubject.h"


#include "ibuffer.h"
#include "icontrolmodule.h"
#include "iclipplane.h"
#include "ierror.h"
#include "ieventobserver.h"
#include "ilookuptable.h"
#include "ipalette.h"
#include "ireplicatedactor.h"
#include "iviewmodule.h"
#include "iviewsubjectobserver.h"
#include "iviewsubjectreplicator.h"

#include <vtkProperty.h>

//
//  Templates
//
#include "iarraytemplate.h"
#include "ibuffertemplate.h"


#include "iparameter.h"
using namespace iParameter;


IVIEWSUBJECT_DEFINE_TYPE(iSolidViewSubject,SolidViewSubject,-svo);

//
//  Inherited keys
//
IOBJECT_DEFINE_KEY(iSolidViewSubject,Color,c,Color,0);
IOBJECT_DEFINE_KEY(iSolidViewSubject,Opacity,o,Float,0);
IOBJECT_DEFINE_KEY(iSolidViewSubject,Palette,p,Int,0);


//
// Main class
//
iSolidViewSubject::iSolidViewSubject(iViewModule *vm, const iDataType &type, const iString &name, int numActors, unsigned int flags) : iViewSubject(vm,type,name,flags) , mActors(0), mOpacities(0.0), mPalettes(0), mColors(iColor()), mNoColor((flags&_ViewSubjectNoColor)!=0U), mNoOpacity((flags&_ViewSubjectNoOpacity)!=0U), mNoPalette((flags&_ViewSubjectNoPalette)!=0U)
{
	int i;

	if(numActors < 1)
	{
		IERROR_FINISH("SolidViewSubject must have at least 1 Actor.");
	}

	iAbortRenderEventObserver *obsAbortRender = this->GetViewModule()->GetAbortRenderEventObserver();

	iActor *a;
	for(i=0; i<numActors; i++)
	{
		mOpacities.Add(1.0f);
		mPalettes.Add(1);
		mColors.Add(mDefaultColor);
		if(mNoReplicating)
		{
			a = iActor::New(); IERROR_ASSERT(a);
		}
		else
		{
			a = iReplicatedActor::New(mReplicator); IERROR_ASSERT(a);
		}
		a->SetScalarVisibility(false);
		a->GetLookupTable()->SetPalette(this->GetViewModule()->GetControlModule()->GetPalette(0));
		a->GetLookupTable()->SetColor(mDefaultColor);
		a->ColorByArrayComponent(0,0); 
		a->SetPosition(0.0,0.0,0.0);
		a->VisibilityOff();
		a->GetProperty()->SetOpacity(1.0);
		a->GetProperty()->SetInterpolationToGouraud();
		a->GetProperty()->SetColor(mDefaultColor.ToVTK());
		a->AddMapperObserver(vtkCommand::ProgressEvent,obsAbortRender);
		this->GetViewModule()->AddObject(a);
		mActors.Add(a);
	}

	this->UpdateMaterialProperties();

	//
	//  Add observer to keep information about this object
	//
	mActors[0]->AddObserver(vtkCommand::UserEvent,mObjectObserver);
}


iSolidViewSubject::~iSolidViewSubject()
{
	int i;
	for(i=0; i<mActors.Size(); i++)
	{
		this->GetViewModule()->RemoveObject(mActors[i]);
		mActors[i]->Delete();
	}
}


void iSolidViewSubject::UpdateLookupTables()
{
	int i;

	for(i=0; i<mActors.Size(); i++)
	{
		mActors[i]->GetLookupTable()->Modified();
	}
}


void iSolidViewSubject::Reset()
{
	int i;
	this->ResetInput();
	this->SyncWithData(this->RequestAll());
	this->ResetPipeline();
	//
	//  Un-initialize, if needed
	//
	if(mIsInitialized)
	{
		this->ShowColorBars(false);
		for(i=0; i<mActors.Size(); i++) mActors[i]->VisibilityOff();
		mIsInitialized = false;
	}
}


void iSolidViewSubject::ShowClipPlane(bool s)
{
	int i;

	if(s && !mClipPlaneOn)
	{
		for(i=0; i<mActors.Size(); i++) mActors[i]->AddClippingPlane(this->GetViewModule()->GetClipPlane());
		mClipPlaneOn = true;
	} 
	if(!s && mClipPlaneOn) 
	{
		for(i=0; i<mActors.Size(); i++) mActors[i]->RemoveClippingPlane(this->GetViewModule()->GetClipPlane());
		mClipPlaneOn = false;
	}
	this->ClearCache();
}


void iSolidViewSubject::ShowColorBars(bool show)
{
	if(mNoPalette || !this->IsVisible()) return;
	this->ShowColorBarsBody(show);
	this->ClearCache();
}


void iSolidViewSubject::SetOpacity(int l, float o)
{ 
	if(mNoOpacity || o<0.0f || o>1.0f) return;

	if(l>=0 && l<mActors.Size())
	{
		mActors[l]->GetProperty()->SetOpacity(o);
		mOpacities[l] = o;
		this->ClearCache();
	}
}


void iSolidViewSubject::SetColor(int l, const iColor &c)
{
	if(mNoColor) return;

	if(l>=0 && l<mActors.Size())
	{
		mActors[l]->GetProperty()->SetColor(c.ToVTK());
		mActors[l]->GetLookupTable()->SetColor(c);
		mColors[l] = c;
		this->ClearCache();
	}
}


void iSolidViewSubject::SetPalette(int l, int p)
{
	int n = this->GetViewModule()->GetControlModule()->GetNumberOfPalettes();
	if(!mNoPalette && l>=0 && l<mActors.Size() && p>-n && p<n && (p!=0 || !mNoColor))
	{
		this->ShowColorBars(false);
		mPalettes[l] = p;
		this->ShowColorBars(true);
		mActors[l]->GetLookupTable()->SetPalette(this->GetViewModule()->GetControlModule()->GetPalette(iMath::Abs(p)-1));
		mActors[l]->GetLookupTable()->SetReversed(p < 0);
		this->ClearCache();
	}
}


void iSolidViewSubject::UpdateMaterialProperties()
{
	int i;
	vtkProperty *prop;

	if(mShading)
	{
		for(i=0; i<mActors.Size(); i++)
		{
			prop = mActors[i]->GetProperty();
			prop->SetAmbient(mAmbient);
			prop->SetDiffuse(mDiffuse);
			prop->SetSpecular(mSpecular);
			prop->SetSpecularPower(mSpecularPower);
		}
	}
	else
	{
		for(i=0; i<mActors.Size(); i++)
		{
			prop = mActors[i]->GetProperty();
			prop->SetAmbient(1.0);
			prop->SetDiffuse(0.0);
			prop->SetSpecular(0.0);
			prop->SetSpecularPower(0.0);
		}
	}
}


bool iSolidViewSubject::IsVisible() const
{
	return (mActors[0]->GetVisibility() != 0);
}


//
//  Two functions used in saving/restoring the state and in creating new instances with
//
void iSolidViewSubject::ViewSubjectPackStateBody(iString &s) const
{
	if(!mNoColor) this->PackValue(s,KeyColor(),mColors.Data(),mColors.Size());
	if(!mNoOpacity) this->PackValue(s,KeyOpacity(),mOpacities.Data(),mOpacities.Size());
	if(!mNoPalette) this->PackValue(s,KeyPalette(),mPalettes.Data(),mPalettes.Size());

	this->SolidViewSubjectPackStateBody(s);
}


void iSolidViewSubject::ViewSubjectUnPackStateBody(const iString &s)
{
	static iBuffer<int> ibuf;
	static iBuffer<float> fbuf;
	static iBuffer<iColor> cbuf;
	int i;

	if(!mNoColor)
	{
		cbuf.Fill(mColors.Data(),mColors.Size());
		if(this->UnPackValue(s,KeyColor(),cbuf,mColors.Size())) for(i=0; i<mColors.Size(); i++) this->SetColor(i,cbuf[i]);
	}

	if(!mNoOpacity)
	{
		fbuf.Fill(mOpacities.Data(),mOpacities.Size());
		if(this->UnPackValue(s,KeyOpacity(),fbuf,mOpacities.Size())) for(i=0; i<mOpacities.Size(); i++) this->SetOpacity(i,fbuf[i]);
	}

	if(!mNoPalette)
	{
		ibuf.Fill(mPalettes.Data(),mPalettes.Size());
		if(this->UnPackValue(s,KeyPalette(),ibuf,mPalettes.Size())) for(i=0; i<mPalettes.Size(); i++) this->SetPalette(i,ibuf[i]);
	}

	this->SolidViewSubjectUnPackStateBody(s);
}

==> ifrit_3.2.0.orig.tar.gz: ifrit-3.2.0/core/isolidviewsubject.cpp <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/


#include "isolidviewsubject.h"


#include "ibuffer.h"
#include "icommoneventobservers.h"
#include "icontrolmodule.h"
#include "iclipplane.h"
#include "ierror.h"
#include "ilookuptable.h"
#include "ipalette.h"
#include "ireplicatedactor.h"
#include "iviewmodule.h"
#include "iviewsubjectobserver.h"
#include "iviewsubjectreplicator.h"

#include <vtkProperty.h>

//
//  Templates
//
#include "iarraytemplate.h"
#include "ibuffertemplate.h"


#include "iparameter.h"
using namespace iParameter;


IVIEWSUBJECT_DEFINE_TYPE(iSolidViewSubject,SolidViewSubject,-svo);

//
//  Inherited keys
//
IOBJECT_DEFINE_KEY(iSolidViewSubject,Color,c,Color,0);
IOBJECT_DEFINE_KEY(iSolidViewSubject,Opacity,o,Float,0);
IOBJECT_DEFINE_KEY(iSolidViewSubject,Palette,p,Int,0);


//
// Main class
//
iSolidViewSubject::iSolidViewSubject(iViewModule *vm, const iDataType &type, const iString &name, int numActors, unsigned int flags) : iViewSubject(vm,type,name,flags), mNoColor((flags&_ViewSubjectNoColor)!=0U), mNoOpacity((flags&_ViewSubjectNoOpacity)!=0U), mNoPalette((flags&_ViewSubjectNoPalette)!=0U), mSameColor((flags&_ViewSubjectSameColor)!=0U), mSameOpacity((flags&_ViewSubjectSameOpacity)!=0U), mSamePalette((flags&_ViewSubjectSamePalette)!=0U)
{
	int i;

	if(numActors < 1)
	{
		IERROR_FATAL("SolidViewSubject must have at least 1 Actor.");
	}

	iAbortRenderEventObserver *obsAbortRender = this->GetViewModule()->GetAbortRenderEventObserver();

	iActor *a;
	for(i=0; i<numActors; i++)
	{
		mOpacities.Add(1.0f);
		mPalettes.Add(1);
		mColors.Add(mDefaultColor);
		if(mNoReplicating)
		{
			a = iActor::New(); IERROR_ASSERT(a);
		}
		else
		{
			a = iReplicatedActor::New(mReplicator); IERROR_ASSERT(a);
		}
		a->SetScalarVisibility(false);
		a->GetLookupTable()->SetPalette(this->GetViewModule()->GetControlModule()->GetPalette(0));
		a->GetLookupTable()->SetColor(mDefaultColor);
		a->ColorByArrayComponent(0,0); 
		a->SetPosition(0.0,0.0,0.0);
		a->VisibilityOff();
		a->GetProperty()->SetOpacity(1.0);
		a->GetProperty()->SetInterpolationToGouraud();
		a->GetProperty()->SetColor(mDefaultColor.ToVTK());
		a->AddMapperObserver(vtkCommand::ProgressEvent,obsAbortRender);
		this->GetViewModule()->AddObject(a);
		mActors.Add(a);
	}

	this->UpdateMaterialProperties();

	//
	//  Add observer to keep information about this object
	//
	mActors[0]->AddObserver(vtkCommand::UserEvent,mObjectObserver);
}


iSolidViewSubject::~iSolidViewSubject()
{
	int i;
	for(i=0; i<mActors.Size(); i++)
	{
		this->GetViewModule()->RemoveObject(mActors[i]);
		mActors[i]->Delete();
	}
}


void iSolidViewSubject::UpdateLookupTables()
{
	int i;

	for(i=0; i<mActors.Size(); i++)
	{
		mActors[i]->GetLookupTable()->Modified();
	}
}


void iSolidViewSubject::Reset()
{
	this->SyncWithData(this->RequestAll());
	this->ResetPipeline();
	//
	//  Un-initialize, if needed
	//
	if(mIsInitialized)
	{
		this->ShowColorBars(false);
		int i;
		for(i=0; i<mActors.Size(); i++) mActors[i]->VisibilityOff();
		mIsInitialized = false;
	}
}


void iSolidViewSubject::ShowClipPlane(bool s)
{
	int i;

	if(s && !mClipPlaneOn)
	{
		for(i=0; i<mActors.Size(); i++) mActors[i]->AddClippingPlane(this->GetViewModule()->GetClipPlane());
		mClipPlaneOn = true;
	} 
	if(!s && mClipPlaneOn) 
	{
		for(i=0; i<mActors.Size(); i++) mActors[i]->RemoveClippingPlane(this->GetViewModule()->GetClipPlane());
		mClipPlaneOn = false;
	}
	this->ClearCache();
}


void iSolidViewSubject::ShowColorBars(bool show)
{
	if(mNoPalette || !this->IsVisible()) return;
	this->ShowColorBarsBody(show);
	this->ClearCache();
}


void iSolidViewSubject::SetColor(int l, const iColor &c)
{
	if(mSameColor && l!=0) l = 0;

	if(!mNoColor && l>=0 && l<mActors.Size())
	{
		int i, n = mSameColor ? mActors.Size() : l+1;
		for(i=l; i<n; i++)
		{
			mActors[i]->GetProperty()->SetColor(c.ToVTK());
			mActors[i]->GetLookupTable()->SetColor(c);
			mColors[i] = c;
		}
		this->ClearCache();
	}
}


void iSolidViewSubject::SetOpacity(int l, float o)
{ 
	if(mSameOpacity && l!=0) l = 0;

	if(!mNoOpacity && o>=0.0f && o<=1.0f && l>=0 && l<mActors.Size())
	{
		int i, n = mSameOpacity ? mActors.Size() : l+1;
		for(i=l; i<n; i++)
		{
			mActors[i]->GetProperty()->SetOpacity(o);
			mOpacities[i] = o;
		}
		this->ClearCache();
	}
}


void iSolidViewSubject::SetPalette(int l, int p)
{
	if(mSamePalette && l!=0) l = 0;

	int n = this->GetViewModule()->GetControlModule()->GetNumberOfPalettes();
	if(!mNoPalette && l>=0 && l<mActors.Size() && p>=-n && p<=n && (p!=0 || !mNoColor))
	{
		this->ShowColorBars(false);
		int i, n = mSamePalette ? mActors.Size() : l+1;
		for(i=l; i<n; i++)
		{
			mActors[i]->GetLookupTable()->SetPalette(this->GetViewModule()->GetControlModule()->GetPalette(iMath::Abs(p)-1));
			mActors[i]->GetLookupTable()->SetReversed(p < 0);
			mPalettes[i] = p;
		}
		this->ShowColorBars(true);
		this->ClearCache();
	}
}


void iSolidViewSubject::UpdateMaterialProperties()
{
	int i;
	vtkProperty *prop;

	if(mShading)
	{
		for(i=0; i<mActors.Size(); i++)
		{
			prop = mActors[i]->GetProperty();
			prop->SetAmbient(mAmbient);
			prop->SetDiffuse(mDiffuse);
			prop->SetSpecular(mSpecular);
			prop->SetSpecularPower(mSpecularPower);
		}
	}
	else
	{
		for(i=0; i<mActors.Size(); i++)
		{
			prop = mActors[i]->GetProperty();
			prop->SetAmbient(1.0);
			prop->SetDiffuse(0.0);
			prop->SetSpecular(0.0);
			prop->SetSpecularPower(0.0);
		}
	}
}


bool iSolidViewSubject::IsVisible() const
{
	return (mActors[0]->GetVisibility() != 0);
}


//
//  Two functions used in saving/restoring the state and in creating new instances with
//
void iSolidViewSubject::ViewSubjectPackStateBody(iString &s) const
{
	if(!mNoColor) this->PackValue(s,KeyColor(),mColors.Data(),mSameColor?1:mColors.Size());
	if(!mNoOpacity) this->PackValue(s,KeyOpacity(),mOpacities.Data(),mSameOpacity?1:mOpacities.Size());
	if(!mNoPalette) this->PackValue(s,KeyPalette(),mPalettes.Data(),mSamePalette?1:mPalettes.Size());

	this->SolidViewSubjectPackStateBody(s);
}


void iSolidViewSubject::ViewSubjectUnPackStateBody(const iString &s)
{
	static iBuffer<int> ibuf;
	static iBuffer<float> fbuf;
	static iBuffer<iColor> cbuf;
	int i, n;

	if(!mNoColor)
	{
		n = mSameColor ? 1 : mColors.Size();
		cbuf.Fill(mColors.Data(),n);
		if(this->UnPackValue(s,KeyColor(),cbuf,n)) for(i=0; i<n; i++) this->SetColor(i,cbuf[i]);
	}

	if(!mNoOpacity)
	{
		n = mSameOpacity ? 1 : mOpacities.Size();
		fbuf.Fill(mOpacities.Data(),n);
		if(this->UnPackValue(s,KeyOpacity(),fbuf,n)) for(i=0; i<n; i++) this->SetOpacity(i,fbuf[i]);
	}

	if(!mNoPalette)
	{
		n = mSamePalette ? 1 : mPalettes.Size();
		ibuf.Fill(mPalettes.Data(),n);
		if(this->UnPackValue(s,KeyPalette(),ibuf,n)) for(i=0; i<n; i++) this->SetPalette(i,ibuf[i]);
	}

	this->SolidViewSubjectUnPackStateBody(s);
}
